const { app, BrowserWindow, Menu, ipcMain, dialog } = require('electron');
const path = require('path');

// Handle creating/removing shortcuts on Windows when installing/uninstalling.
if (require('electron-squirrel-startup')) {
    app.quit();
}

let mainWindow;

const createWindow = () => {
    // Create the browser window.
    mainWindow = new BrowserWindow({
        width: 1000,
        height: 800,
        backgroundColor: 'black',
        frame: false,
        webPreferences: {
            preload: path.join(__dirname, 'preload.js'),
            nodeIntegration: true,
            contextIsolation: false,
            additionalArguments: [`EDS-ENV=${app.isPackaged}`],
        },
    });

    // and load the index.html of the app.
    if (MAIN_WINDOW_VITE_DEV_SERVER_URL) {
        mainWindow.loadURL(MAIN_WINDOW_VITE_DEV_SERVER_URL);
    } else {
        mainWindow.loadFile(path.join(__dirname, `../renderer/${MAIN_WINDOW_VITE_NAME}/index.html`));
    }

    // Open the DevTools.
    if (!app.isPackaged) {
        mainWindow.webContents.openDevTools();
    }
};

// Set up close window handler for our custom close button.
const closeWindow = (event) => {
    const webContents = event.sender;
    const window = BrowserWindow.fromWebContents(webContents);
    window?.close();
}

// Set up settings page handler.
const loadSettings = (event) => {
    const webContents = event.sender;
    const window = BrowserWindow.fromWebContents(webContents);

    if (SETTINGS_WINDOW_VITE_DEV_SERVER_URL) {
        window.loadURL(`${SETTINGS_WINDOW_VITE_DEV_SERVER_URL}/settings.html`);
    } else {
        window.loadFile(path.join(__dirname, `../renderer/${SETTINGS_WINDOW_VITE_NAME}/settings.html`));
    }
}

// Set up main page handler.
const loadMain = (event) => {
    const webContents = event.sender;
    const window = BrowserWindow.fromWebContents(webContents);

    if (MAIN_WINDOW_VITE_DEV_SERVER_URL) {
        mainWindow.loadURL(MAIN_WINDOW_VITE_DEV_SERVER_URL);
    } else {
        mainWindow.loadFile(path.join(__dirname, `../renderer/${MAIN_WINDOW_VITE_NAME}/index.html`));
    }
}

// Set up file picker handler.
const selectMatrixFile = async () => {
    const { canceled, filePaths } = await dialog.showOpenDialog(mainWindow, {
        filters: [{name: 'Matrix File', extensions: ['xml', 'ini']}],
        properties: ['openFile', 'showHiddenFiles'],
    });

    if (canceled) {
        return;
    } else {
        return filePaths[0];
    }

}

// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
// Some APIs can only be used after this event occurs.
app.on('ready', () => {
    ipcMain.on('CLOSE_WINDOW', closeWindow);
    ipcMain.on('LOAD_SETTINGS', loadSettings);
    ipcMain.on('LOAD_MAIN', loadMain);
    ipcMain.handle('SELECT_MATRIX_FILE', selectMatrixFile);
    createWindow();
});

// Quit when all windows are closed, except on macOS. There, it's common
// for applications and their menu bar to stay active until the user quits
// explicitly with Cmd + Q.
app.on('window-all-closed', () => {
    if (process.platform !== 'darwin') {
        app.quit();
    }
});

app.on('activate', () => {
    // On OS X it's common to re-create a window in the app when the
    // dock icon is clicked and there are no other windows open.
    if (BrowserWindow.getAllWindows().length === 0) {
        createWindow();
    }
});

// In this file you can include the rest of your app's specific main process
// code. You can also put them in separate files and import them here.